@@ -58,12 +58,6 @@ static struct {
 		DBUS_TYPE_STRING, "IPv4.Broadcast" },
 	{ CONNMAN_PROPERTY_ID_IPV4_NAMESERVER,
 		DBUS_TYPE_STRING, "IPv4.Nameserver" },
-
-	{ CONNMAN_PROPERTY_ID_WIFI_SECURITY,
-		DBUS_TYPE_STRING, "WiFi.Security" },
-	{ CONNMAN_PROPERTY_ID_WIFI_PASSPHRASE,
-		DBUS_TYPE_STRING, "WiFi.Passphrase" },
-
 	{ }
 };
 
@@ -120,61 +114,13 @@ static const char *type2string(enum connman_element_type type)
 		return "zeroconf";
 	case CONNMAN_ELEMENT_TYPE_CONNECTION:
 		return "connection";
+	case CONNMAN_ELEMENT_TYPE_VENDOR:
+		return "vendor";
 	}
 
 	return NULL;
 }
 
-static const char *subtype2string(enum connman_element_subtype type)
-{
-	switch (type) {
-	case CONNMAN_ELEMENT_SUBTYPE_UNKNOWN:
-		return "unknown";
-	case CONNMAN_ELEMENT_SUBTYPE_FAKE:
-		return "fake";
-	case CONNMAN_ELEMENT_SUBTYPE_ETHERNET:
-		return "ethernet";
-	case CONNMAN_ELEMENT_SUBTYPE_WIFI:
-		return "wifi";
-	case CONNMAN_ELEMENT_SUBTYPE_WIMAX:
-		return "wimax";
-	case CONNMAN_ELEMENT_SUBTYPE_CELLULAR:
-		return "cellular";
-	case CONNMAN_ELEMENT_SUBTYPE_BLUETOOTH:
-		return "bluetooth";
-	}
-
-	return NULL;
-}
-
-const char *__connman_element_policy2string(enum connman_element_policy policy)
-{
-	switch (policy) {
-	case CONNMAN_ELEMENT_POLICY_UNKNOWN:
-		return "unknown";
-	case CONNMAN_ELEMENT_POLICY_IGNORE:
-		return "ignore";
-	case CONNMAN_ELEMENT_POLICY_AUTO:
-		return "auto";
-	case CONNMAN_ELEMENT_POLICY_ASK:
-		return "ask";
-	}
-
-	return NULL;
-}
-
-enum connman_element_policy __connman_element_string2policy(const char *policy)
-{
-	if (strcasecmp(policy, "ignore") == 0)
-		return CONNMAN_ELEMENT_POLICY_IGNORE;
-	else if (strcasecmp(policy, "auto") == 0)
-		return CONNMAN_ELEMENT_POLICY_AUTO;
-	else if (strcasecmp(policy, "ask") == 0)
-		return CONNMAN_ELEMENT_POLICY_ASK;
-	else
-		return CONNMAN_ELEMENT_POLICY_UNKNOWN;
-}
-
 const char *__connman_ipv4_method2string(enum connman_ipv4_method method)
 {
 	switch (method) {
@@ -203,6 +149,7 @@ enum connman_ipv4_method __connman_ipv4_string2method(const char *method)
 		return CONNMAN_IPV4_METHOD_UNKNOWN;
 }
 
+#if 0
 static void append_property(DBusMessageIter *dict,
 				struct connman_property *property)
 {
@@ -310,306 +257,29 @@ static void set_common_property(struct connman_element *element,
 
 	__connman_element_unlock(element);
 }
+#endif
 
-static void emit_enabled_signal(DBusConnection *conn,
+static void emit_element_signal(DBusConnection *conn, const char *member,
 					struct connman_element *element)
 {
 	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *iface, *key;
 
-	DBG("conn %p", conn);
+	if (__connman_debug_enabled() == FALSE)
+		return;
+
+	DBG("conn %p member %s", conn, member);
 
 	if (element == NULL)
 		return;
 
-	switch (element->type) {
-	case CONNMAN_ELEMENT_TYPE_DEVICE:
-		iface = CONNMAN_DEVICE_INTERFACE;
-		key = "Powered";
-		break;
-	case CONNMAN_ELEMENT_TYPE_NETWORK:
-		iface = CONNMAN_NETWORK_INTERFACE;
-		key = "Connected";
-		break;
-	case CONNMAN_ELEMENT_TYPE_CONNECTION:
-		iface = CONNMAN_CONNECTION_INTERFACE;
-		key = "Default";
-		break;
-	default:
-		return;
-	}
-
 	signal = dbus_message_new_signal(element->path,
-						iface, "PropertyChanged");
+					CONNMAN_DEBUG_INTERFACE, member);
 	if (signal == NULL)
 		return;
 
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN,
-							&element->enabled);
-	dbus_message_iter_close_container(&entry, &value);
-
 	g_dbus_send_message(conn, signal);
 }
 
-static void emit_scanning_signal(DBusConnection *conn,
-					struct connman_element *element)
-{
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *key = "Scanning";
-
-	DBG("conn %p", conn);
-
-	if (element == NULL)
-		return;
-
-	if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
-		return;
-
-	signal = dbus_message_new_signal(element->path,
-				CONNMAN_DEVICE_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN,
-							&element->scanning);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(conn, signal);
-}
-
-static DBusMessage *do_enable(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-
-	DBG("conn %p", conn);
-
-	if (element->enabled == TRUE)
-		return __connman_error_failed(msg);
-
-	if (element->driver && element->driver->enable) {
-		DBG("Calling enable callback");
-		if (element->driver->enable(element) < 0)
-			return __connman_error_failed(msg);
-	}
-
-	element->enabled = TRUE;
-
-	emit_enabled_signal(connection, element);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *do_disable(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-
-	DBG("conn %p", conn);
-
-	if (element->enabled == FALSE)
-		return __connman_error_failed(msg);
-
-	if (element->driver && element->driver->disable) {
-		DBG("Calling disable callback");
-		if (element->driver->disable(element) < 0)
-			return __connman_error_failed(msg);
-	}
-
-	element->enabled = FALSE;
-
-	emit_enabled_signal(connection, element);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *network_get_properties(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-	DBusMessage *reply;
-	DBusMessageIter array, dict;
-
-	DBG("conn %p", conn);
-
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
-
-	dbus_message_iter_init_append(reply, &array);
-
-	dbus_message_iter_open_container(&array, DBUS_TYPE_ARRAY,
-			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
-			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
-			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
-
-	if (element->parent)
-		connman_dbus_dict_append_variant(&dict, "Device",
-				DBUS_TYPE_OBJECT_PATH, &element->parent->path);
-
-	connman_dbus_dict_append_variant(&dict, "Available",
-					DBUS_TYPE_BOOLEAN, &element->available);
-
-	connman_dbus_dict_append_variant(&dict, "Connected",
-					DBUS_TYPE_BOOLEAN, &element->enabled);
-
-	connman_dbus_dict_append_variant(&dict, "Remember",
-					DBUS_TYPE_BOOLEAN, &element->remember);
-
-	add_common_properties(element, &dict);
-
-	dbus_message_iter_close_container(&array, &dict);
-
-	return reply;
-}
-
-static DBusMessage *network_set_property(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-	DBusMessageIter iter;
-	DBusMessageIter value;
-	const char *name;
-
-	DBG("conn %p", conn);
-
-	if (dbus_message_iter_init(msg, &iter) == FALSE)
-		return __connman_error_invalid_arguments(msg);
-
-	dbus_message_iter_get_basic(&iter, &name);
-	dbus_message_iter_next(&iter);
-	dbus_message_iter_recurse(&iter, &value);
-
-	if (__connman_security_check_privileges(msg) < 0)
-		return __connman_error_permission_denied(msg);
-
-	if (g_str_equal(name, "Remember") == TRUE) {
-		dbus_message_iter_get_basic(&value, &element->remember);
-	} else if (g_str_equal(name, "WiFi.Passphrase") == TRUE) {
-		const char *str;
-
-		dbus_message_iter_get_basic(&value, &str);
-		g_free(element->wifi.passphrase);
-		element->wifi.passphrase = g_strdup(str);
-	} else
-		set_common_property(element, name, &value);
-
-	__connman_element_store(element);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *connection_get_properties(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-	DBusMessage *reply;
-	DBusMessageIter array, dict;
-	const char *str;
-
-	DBG("conn %p", conn);
-
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
-
-	dbus_message_iter_init_append(reply, &array);
-
-	dbus_message_iter_open_container(&array, DBUS_TYPE_ARRAY,
-			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
-			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
-			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
-
-	str = subtype2string(element->subtype);
-	if (str != NULL)
-		connman_dbus_dict_append_variant(&dict, "Type",
-						DBUS_TYPE_STRING, &str);
-
-	if (element->devname != NULL)
-		connman_dbus_dict_append_variant(&dict, "Interface",
-					DBUS_TYPE_STRING, &element->devname);
-
-	if (element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIFI ||
-			element->subtype == CONNMAN_ELEMENT_SUBTYPE_WIMAX)
-		connman_dbus_dict_append_variant(&dict, "Strength",
-					DBUS_TYPE_BYTE, &element->strength);
-
-	connman_dbus_dict_append_variant(&dict, "Default",
-					DBUS_TYPE_BOOLEAN, &element->enabled);
-
-	add_common_properties(element, &dict);
-
-	dbus_message_iter_close_container(&array, &dict);
-
-	return reply;
-}
-
-static DBusMessage *connection_set_property(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_element *element = data;
-	DBusMessageIter iter, value;
-	const char *name;
-
-	DBG("conn %p", conn);
-
-	if (dbus_message_iter_init(msg, &iter) == FALSE)
-		return __connman_error_invalid_arguments(msg);
-
-	dbus_message_iter_get_basic(&iter, &name);
-	dbus_message_iter_next(&iter);
-	dbus_message_iter_recurse(&iter, &value);
-
-	if (__connman_security_check_privileges(msg) < 0)
-		return __connman_error_permission_denied(msg);
-
-	if (g_str_equal(name, "Default") == TRUE) {
-		dbus_bool_t enabled;
-
-		dbus_message_iter_get_basic(&value, &enabled);
-
-		if (enabled == TRUE)
-			return do_enable(conn, msg, element);
-		else
-			return do_disable(conn, msg, element);
-	}
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static GDBusMethodTable network_methods[] = {
-	{ "GetProperties", "",   "a{sv}", network_get_properties },
-	{ "SetProperty",   "sv", "",      network_set_property   },
-	{ "Connect",       "",   "",      do_enable              },
-	{ "Disconnect",    "",   "",      do_disable             },
-	{ },
-};
-
-static GDBusMethodTable connection_methods[] = {
-	{ "GetProperties", "",   "a{sv}", connection_get_properties },
-	{ "SetProperty",   "sv", "",      connection_set_property   },
-	{ },
-};
-
-static GDBusSignalTable element_signals[] = {
-	{ "PropertyChanged", "sv" },
-	{ },
-};
-
 struct foreach_data {
 	enum connman_element_type type;
 	element_cb_t callback;
@@ -676,6 +346,14 @@ static gboolean append_path(GNode *node, gpointer user_data)
 					filter->type != element->type)
 		return FALSE;
 
+	if (filter->type == CONNMAN_ELEMENT_TYPE_DEVICE &&
+			__connman_device_has_driver(element->device) == FALSE)
+		return FALSE;
+
+	if (filter->type == CONNMAN_ELEMENT_TYPE_NETWORK &&
+			__connman_network_has_driver(element->network) == FALSE)
+		return FALSE;
+
 	dbus_message_iter_append_basic(filter->iter,
 				DBUS_TYPE_OBJECT_PATH, &element->path);
 
@@ -749,6 +427,30 @@ int __connman_element_count(struct connman_element *element,
 	return data.count;
 }
 
+const char *__connman_element_get_device(struct connman_element *element)
+{
+	if (element->type == CONNMAN_ELEMENT_TYPE_DEVICE &&
+						element->device != NULL)
+		return element->path;
+
+	if (element->parent == NULL)
+		return NULL;
+
+	return __connman_element_get_device(element->parent);
+}
+
+const char *__connman_element_get_network(struct connman_element *element)
+{
+	if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK &&
+						element->network != NULL)
+		return element->path;
+
+	if (element->parent == NULL)
+		return NULL;
+
+	return __connman_element_get_network(element->parent);
+}
+
 static gint compare_priority(gconstpointer a, gconstpointer b)
 {
 	const struct connman_driver *driver1 = a;
@@ -763,33 +465,13 @@ static gboolean match_driver(struct connman_element *element,
 	if (element->type == CONNMAN_ELEMENT_TYPE_ROOT)
 		return FALSE;
 
-	if (element->type != driver->type &&
-			driver->type != CONNMAN_ELEMENT_TYPE_UNKNOWN)
-		return FALSE;
-
-	if (element->subtype == driver->subtype ||
-			driver->subtype == CONNMAN_ELEMENT_SUBTYPE_UNKNOWN)
+	if (element->type == driver->type ||
+			driver->type == CONNMAN_ELEMENT_TYPE_UNKNOWN)
 		return TRUE;
 
 	return FALSE;
 }
 
-static void enable_element(struct connman_element *element)
-{
-	if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
-		return;
-
-	if (element->policy != CONNMAN_ELEMENT_POLICY_AUTO)
-		return;
-
-	if (element->driver && element->driver->enable) {
-		if (element->driver->enable(element) == 0) {
-			element->enabled = TRUE;
-			emit_enabled_signal(connection, element);
-		}
-	}
-}
-
 static gboolean probe_driver(GNode *node, gpointer data)
 {
 	struct connman_element *element = node->data;
@@ -804,8 +486,6 @@ static gboolean probe_driver(GNode *node, gpointer data)
 		__connman_element_lock(element);
 		element->driver = driver;
 		__connman_element_unlock(element);
-
-		enable_element(element);
 	}
 
 	return FALSE;
@@ -854,22 +534,6 @@ int connman_driver_register(struct connman_driver *driver)
 	return 0;
 }
 
-static void disable_element(struct connman_element *element)
-{
-	if (element->policy != CONNMAN_ELEMENT_POLICY_AUTO)
-		return;
-
-	if (element->enabled == FALSE)
-		return;
-
-	if (element->driver && element->driver->disable) {
-		if (element->driver->disable(element) == 0) {
-			element->enabled = FALSE;
-			emit_enabled_signal(connection, element);
-		}
-	}
-}
-
 static gboolean remove_driver(GNode *node, gpointer data)
 {
 	struct connman_element *element = node->data;
@@ -878,8 +542,6 @@ static gboolean remove_driver(GNode *node, gpointer data)
 	DBG("element %p name %s", element, element->name);
 
 	if (element->driver == driver) {
-		disable_element(element);
-
 		if (driver->remove)
 			driver->remove(element);
 
@@ -931,9 +593,6 @@ struct connman_element *connman_element_create(const char *name)
 
 	element->name    = g_strdup(name);
 	element->type    = CONNMAN_ELEMENT_TYPE_UNKNOWN;
-	element->subtype = CONNMAN_ELEMENT_SUBTYPE_UNKNOWN;
-	element->state   = CONNMAN_ELEMENT_STATE_CLOSED;
-	element->policy  = CONNMAN_ELEMENT_POLICY_AUTO;
 	element->index   = -1;
 	element->enabled = FALSE;
 
@@ -991,7 +650,6 @@ void connman_element_unref(struct connman_element *element)
 		g_free(element->ipv4.broadcast);
 		g_free(element->ipv4.nameserver);
 		g_free(element->devname);
-		g_free(element->devpath);
 		g_free(element->path);
 		g_free(element->name);
 		g_free(element);
@@ -1037,59 +695,6 @@ int connman_element_add_static_property(struct connman_element *element,
 	return 0;
 }
 
-static void emit_property_changed(DBusConnection *conn,
-				struct connman_element *element,
-				const char *name, int type, const void *data)
-{
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *iface, *sig;
-
-	DBG("conn %p", conn);
-
-	switch (element->type) {
-	case CONNMAN_ELEMENT_TYPE_DEVICE:
-		iface = CONNMAN_DEVICE_INTERFACE;
-		break;
-	case CONNMAN_ELEMENT_TYPE_NETWORK:
-		iface = CONNMAN_NETWORK_INTERFACE;
-		break;
-	case CONNMAN_ELEMENT_TYPE_CONNECTION:
-		iface = CONNMAN_CONNECTION_INTERFACE;
-		break;
-	default:
-		return;
-	}
-
-	signal = dbus_message_new_signal(element->path,
-						iface, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &name);
-
-	switch (type) {
-	case DBUS_TYPE_STRING:
-		sig = DBUS_TYPE_STRING_AS_STRING;
-		break;
-	case DBUS_TYPE_BYTE:
-		sig = DBUS_TYPE_BYTE_AS_STRING;
-		break;
-	default:
-		sig = DBUS_TYPE_VARIANT_AS_STRING;
-		break;
-	}
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-							sig, &value);
-	dbus_message_iter_append_basic(&value, type, data);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(conn, signal);
-}
-
 int connman_element_set_static_property(struct connman_element *element,
 				const char *name, int type, const void *value)
 {
@@ -1128,8 +733,6 @@ int connman_element_set_static_property(struct connman_element *element,
 
 	__connman_element_unlock(element);
 
-	emit_property_changed(connection, element, name, type, value);
-
 	return 0;
 }
 
@@ -1363,18 +966,6 @@ int connman_element_set_property(struct connman_element *element,
 		element->ipv4.nameserver = g_strdup(*((const char **) value));
 		__connman_element_unlock(element);
 		break;
-	case CONNMAN_PROPERTY_ID_WIFI_SECURITY:
-		__connman_element_lock(element);
-		g_free(element->wifi.security);
-		element->wifi.security = g_strdup(*((const char **) value));
-		__connman_element_unlock(element);
-		break;
-	case CONNMAN_PROPERTY_ID_WIFI_PASSPHRASE:
-		__connman_element_lock(element);
-		g_free(element->wifi.passphrase);
-		element->wifi.passphrase = g_strdup(*((const char **) value));
-		__connman_element_unlock(element);
-		break;
 	default:
 		return -EINVAL;
 	}
@@ -1389,6 +980,14 @@ int connman_element_get_value(struct connman_element *element,
 		return -EINVAL;
 
 	switch (id) {
+	case CONNMAN_PROPERTY_ID_IPV4_METHOD:
+		if (element->ipv4.method == CONNMAN_IPV4_METHOD_UNKNOWN)
+			return connman_element_get_value(element->parent,
+								id, value);
+		__connman_element_lock(element);
+		*((const char **) value) = __connman_ipv4_method2string(element->ipv4.method);
+		__connman_element_unlock(element);
+		break;
 	case CONNMAN_PROPERTY_ID_IPV4_ADDRESS:
 		if (element->ipv4.address == NULL)
 			return connman_element_get_value(element->parent,
@@ -1429,22 +1028,6 @@ int connman_element_get_value(struct connman_element *element,
 		*((char **) value) = element->ipv4.nameserver;
 		__connman_element_unlock(element);
 		break;
-	case CONNMAN_PROPERTY_ID_WIFI_SECURITY:
-		if (element->wifi.security == NULL)
-			return connman_element_get_value(element->parent,
-								id, value);
-		__connman_element_lock(element);
-		*((char **) value) = element->wifi.security;
-		__connman_element_unlock(element);
-		break;
-	case CONNMAN_PROPERTY_ID_WIFI_PASSPHRASE:
-		if (element->wifi.passphrase == NULL)
-			return connman_element_get_value(element->parent,
-								id, value);
-		__connman_element_lock(element);
-		*((char **) value) = element->wifi.passphrase;
-		__connman_element_unlock(element);
-		break;
 	default:
 		return -EINVAL;
 	}
@@ -1474,6 +1057,10 @@ gboolean connman_element_get_static_property(struct connman_element *element,
 				*((char **) value) = property->value;
 				found = TRUE;
 				break;
+			case DBUS_TYPE_BYTE:
+				memcpy(value, property->value, 1);
+				found = TRUE;
+				break;
 			}
 			break;
 		}
@@ -1481,6 +1068,10 @@ gboolean connman_element_get_static_property(struct connman_element *element,
 
 	__connman_element_unlock(element);
 
+	if (found == FALSE && element->parent != NULL)
+		return connman_element_get_static_property(element->parent,
+								name, value);
+
 	return found;
 }
 
@@ -1545,49 +1136,6 @@ gboolean connman_element_match_static_property(struct connman_element *element,
 	return result;
 }
 
-static void append_networks(struct connman_element *element,
-						DBusMessageIter *entry)
-{
-	DBusMessageIter value, iter;
-	const char *key = "Networks";
-
-	dbus_message_iter_append_basic(entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
-		DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_OBJECT_PATH_AS_STRING,
-								&value);
-
-	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
-				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
-	__connman_element_list(element, CONNMAN_ELEMENT_TYPE_NETWORK, &iter);
-	dbus_message_iter_close_container(&value, &iter);
-
-	dbus_message_iter_close_container(entry, &value);
-}
-
-static void emit_networks_signal(DBusConnection *conn,
-					struct connman_element *device)
-{
-	DBusMessage *signal;
-	DBusMessageIter entry;
-
-	DBG("conn %p", conn);
-
-	if (device == NULL)
-		return;
-
-	signal = dbus_message_new_signal(device->path,
-				CONNMAN_DEVICE_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	append_networks(device, &entry);
-
-	g_dbus_send_message(conn, signal);
-}
-
 static void append_connections(DBusMessageIter *entry)
 {
 	DBusMessageIter value, iter;
@@ -1658,20 +1206,6 @@ static void emit_state_change(DBusConnection *conn, const char *state)
 	g_dbus_send_message(conn, signal);
 }
 
-static void set_signal_strength(struct connman_element *connection)
-{
-	struct connman_element *element = connection;
-
-	while (element != NULL) {
-		if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK) {
-			connection->strength = element->strength;
-			break;
-		}
-
-		element = element->parent;
-	}
-}
-
 static void probe_element(struct connman_element *element)
 {
 	GSList *list;
@@ -1690,8 +1224,6 @@ static void probe_element(struct connman_element *element)
 			__connman_element_lock(element);
 			element->driver = driver;
 			__connman_element_unlock(element);
-
-			enable_element(element);
 			break;
 		}
 	}
@@ -1709,9 +1241,6 @@ static void register_element(gpointer data, gpointer user_data)
 		node = g_node_find(element_root, G_PRE_ORDER,
 					G_TRAVERSE_ALL, element->parent);
 		basepath = element->parent->path;
-
-		if (element->subtype == CONNMAN_ELEMENT_SUBTYPE_UNKNOWN)
-			element->subtype = element->parent->subtype;
 	} else {
 		element->parent = element_root->data;
 
@@ -1727,36 +1256,14 @@ static void register_element(gpointer data, gpointer user_data)
 
 	DBG("element %p path %s", element, element->path);
 
-	__connman_element_load(element);
-
 	g_node_append_data(node, element);
 
-	if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK) {
-		if (g_dbus_register_interface(connection, element->path,
-					CONNMAN_NETWORK_INTERFACE,
-					network_methods, element_signals,
-					NULL, element, NULL) == FALSE)
-			connman_error("Failed to register %s network",
-								element->path);
-		else
-			emit_networks_signal(connection, element->parent);
-	}
-
 	if (element->type == CONNMAN_ELEMENT_TYPE_CONNECTION) {
-		if (g_dbus_register_interface(connection, element->path,
-					CONNMAN_CONNECTION_INTERFACE,
-					connection_methods, element_signals,
-					NULL, element, NULL) == FALSE)
-			connman_error("Failed to register %s connection",
-								element->path);
-		else {
-			set_signal_strength(element);
-			emit_connections_signal(connection);
-			emit_state_change(connection, "online");
-		}
+		emit_connections_signal(connection);
+		emit_state_change(connection, "online");
 	}
 
-	__connman_element_store(element);
+	emit_element_signal(connection, "ElementAdded", element);
 
 	if (started == FALSE)
 		return;
@@ -1804,6 +1311,9 @@ int connman_element_register(struct connman_element *element,
 		}
 	}
 
+	if (element->type == CONNMAN_ELEMENT_TYPE_DHCP)
+		element->ipv4.method = CONNMAN_IPV4_METHOD_DHCP;
+
 	element->parent = parent;
 
 	__connman_element_unlock(element);
@@ -1827,8 +1337,6 @@ static gboolean remove_element(GNode *node, gpointer user_data)
 		g_node_unlink(node);
 
 	if (element->driver) {
-		disable_element(element);
-
 		if (element->driver->remove)
 			element->driver->remove(element);
 
@@ -1845,17 +1353,9 @@ static gboolean remove_element(GNode *node, gpointer user_data)
 					CONNMAN_ELEMENT_TYPE_CONNECTION) == 0)
 			emit_state_change(connection, "offline");
 		emit_connections_signal(connection);
-
-		g_dbus_unregister_interface(connection, element->path,
-						CONNMAN_CONNECTION_INTERFACE);
 	}
 
-	if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK) {
-		emit_networks_signal(connection, element->parent);
-
-		g_dbus_unregister_interface(connection, element->path,
-						CONNMAN_NETWORK_INTERFACE);
-	}
+	emit_element_signal(connection, "ElementRemoved", element);
 
 	connman_element_unref(element);
 
@@ -1891,21 +1391,13 @@ void connman_element_unregister_children(struct connman_element *element)
 static gboolean update_element(GNode *node, gpointer user_data)
 {
 	struct connman_element *element = node->data;
-	struct connman_element *root = user_data;
 
 	DBG("element %p name %s", element, element->name);
 
 	if (element->driver && element->driver->update)
 		element->driver->update(element);
 
-	if (element->type == CONNMAN_ELEMENT_TYPE_CONNECTION &&
-				root->type == CONNMAN_ELEMENT_TYPE_NETWORK) {
-		if (element->strength != root->strength) {
-			element->strength = root->strength;
-			emit_property_changed(connection, element, "Strength",
-					DBUS_TYPE_BYTE, &element->strength);
-		}
-	}
+	emit_element_signal(connection, "ElementUpdated", element);
 
 	return FALSE;
 }
@@ -1931,21 +1423,6 @@ int connman_element_set_enabled(struct connman_element *element,
 
 	element->enabled = enabled;
 
-	emit_enabled_signal(connection, element);
-
-	return 0;
-}
-
-int connman_element_set_scanning(struct connman_element *element,
-							gboolean scanning)
-{
-	if (element->scanning == scanning)
-		return 0;
-
-	element->scanning = scanning;
-
-	emit_scanning_signal(connection, element);
-
 	return 0;
 }
 
@@ -1970,8 +1447,8 @@ int __connman_element_init(DBusConnection *conn, const char *device)
 
 	element_root = g_node_new(element);
 
-	__connman_device_init();
 	__connman_network_init();
+	__connman_device_init();
 
 	return 0;
 }
@@ -2002,6 +1479,10 @@ void __connman_element_start(void)
 
 	started = TRUE;
 
+	__connman_storage_init_device();
+
+	__connman_connection_init();
+	__connman_ipv4_init();
 	__connman_detect_init();
 }
 
@@ -2010,6 +1491,8 @@ void __connman_element_stop(void)
 	DBG("");
 
 	__connman_detect_cleanup();
+	__connman_ipv4_cleanup();
+	__connman_connection_cleanup();
 }
 
 static gboolean free_driver(GNode *node, gpointer data)
@@ -2019,8 +1502,6 @@ static gboolean free_driver(GNode *node, gpointer data)
 	DBG("element %p name %s", element, element->name);
 
 	if (element->driver) {
-		disable_element(element);
-
 		if (element->driver->remove)
 			element->driver->remove(element);
 
@@ -2048,8 +1529,8 @@ void __connman_element_cleanup(void)
 {
 	DBG("");
 
-	__connman_network_cleanup();
 	__connman_device_cleanup();
+	__connman_network_cleanup();
 
 	g_node_traverse(element_root, G_POST_ORDER, G_TRAVERSE_ALL, -1,
 							free_driver, NULL);
